// AUTHENTICATION.JS
// Cole Cassidy - 2022
// Handles server side authentication via JSON Web Tokens

// Imports
const express = require('express');
const router = express.Router();

const { ObjectId } = require('mongodb');

const jwt = require('jsonwebtoken');
const bcrypt = require('bcrypt');
const { hash } = require('bcrypt');

const iota = require('./iota');
const dbconn = require('./dbConn');


const validator = require('./validator');



// ========================================
//        Authentication Functions
// ========================================

// ----- Core -----

// Registers a new user in the database.
// Creates unique entry in both user and auth databases
const regUser = async (user) => {
    
    var result;

    // create doc for user db
    newUsr = {
        "uName": user.uName.toUpperCase(),
        "fName": user.fName,
        "lName": user.lName,
        "rank": 0,
        "joindate": new Date()
    }

    // create doc for auth db
    secUsr = {
        "uName": user.uName.toUpperCase()
    }

    // perform duplicate check against user db
    var check = await checkUserExists(user.uName.toUpperCase());
    if(check) {
        return iota.ERR.UserExists;
    }

    // append hashed password to auth db doc
    secUsr.pWord = await passCrypt(user.pWord);

    // write new user doc to user db
    try {
        result = await dbconn.get().collection("users").insertOne(newUsr);
        result = newUsr;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }

    // generate a user unique token
    token = await makeToken(result);

    // append unique token to auth doc
    secUsr.uid = result._id;
    secUsr.token = [token];

    // write auth doc to db
    dbconn.get().collection("auth").insertOne(secUsr);

    // append new token to result for the registering application
    result.token = token;
    return result;
}

// Handles user authentication.
// Returns a token if user credentials are correct.
const authUser = async (doc) => {
    const userAuth = await dbconn.get().collection("auth").findOne({"uName": doc.uName.toUpperCase()});
    console.log(userAuth);

    if(userAuth == null) return iota.ERR.UserDNE;
    // iota.AUTHCONFIG.SaltRounds;

    const user = await dbconn.get().collection("users").findOne({"_id": userAuth.uid });
    // console.log("A");
    // console.log(user);
    // console.log(await bcrypt.compare(doc.pWord, userAuth.pWord));


    
    if(await bcrypt.compare(doc.pWord, userAuth.pWord)){
        token = await makeToken(user);
        console.log(token);
        user.token = token;
        return user;
    }
    return iota.ERR.Unauthorized;
}

// ----- Support -----

// Validates if user with given uName exists.
const checkUserExists = async (name) => {
    // Returns True if user exists. Otherwise returns false.
    // Also returns true if error occurs in order to protect DB from duplicates.
    try {
        const doc = await dbconn.get().collection("users").findOne({"uName": name.toUpperCase()});
        if(doc != null) return true;
        return false;
    } catch(e) {
        console.error(e);
        return true;
    }
}

// Returns the hash for a given password.
const passCrypt = async (badPass) => {
    // Hashes cleartext password.
    return await bcrypt.hash(badPass, iota.AUTHCONFIG.SaltRounds);
}

// Generates A new signed token.
const makeToken = async (user) => {
    // Creates new token from provided user doc
    const token = jwt.sign(
        user,
        iota.AUTHCONFIG.TokenKey
    );
    return token;
}

// Returns an updated user token.
const updateToken = async (user) => {
    try {
        const newDoc = await dbconn.get().collection("users").findOne({"_id": new ObjectId(user._id)});
        const token = jwt.sign(
            newDoc,
            iota.AUTHCONFIG.TokenKey
        );
        return token;
    } catch(e){
        console.error("Failed Token Update");
        return iota.ERR.Unknown;
    }
}

// ========================================
//              Express Routes
// ========================================

// middleware to log endpoint access
router.use((req, res, next) => {
    console.log("REQ: AUT  IP: "+req.ip);
    next();
});

// ---- GET ----

// API endpoint to return an updated token if current token is valid
// "../auth/update"
router.get('/update', validator.checkToken, async (req, res) => {
    const doc = await updateToken(req.user);
    res.json(doc);
});

// ---- POST ----
// API endpoint to allow users to authenticate 
// "../auth"
router.post('/', async (req, res) => {
    const doc = await authUser(req.body);
    res.json(doc);
});


// API endpoint to allow registration of new user
// "../auth/reg"
router.post('/reg', async (req, res) => {
    const doc = await regUser(req.body);
    res.json(doc);
});

// API endpoint to allow user to authenticate (alternate)
// "../auth/login"
router.post('/login', async (req, res) => {
     console.log(req.body);
     const doc = await authUser(req.body);
     res.json(doc);
});

// EXPORTS
module.exports = {
    router
}