// MINUTES.JS
// Cole Cassidy - 2022
// Contains core user functions and routing

// Imports
const express = require('express');
const { normalizeType } = require('express/lib/utils');
const router = express.Router();

const { ObjectId } = require('mongodb');
const dbconn = require('./dbConn');
const validator = require('./validator');

const iota = require('./iota');

// ========================================
//              DB Functions
// ========================================

// ----- Read -----

// returns a document containing all minutes.
const listAll = async () => {
    try {
        const doc = await dbconn.get().collection("minutes").find().toArray();
        return doc;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }
}

// returns minutes filtered by an organization ID
const minByOrg = async (id) => {
    try {
        console.log(id);
        const doc = await dbconn.get().collection("minutes").find({"oid": new ObjectId(id)}).toArray();
        return doc;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }
}

// returns a minute document with the given ID
const minByID = async (id) => {
    try {
        const doc = await dbconn.get().collection("minutes").findOne({"_id": new ObjectId(id)});
        console.log(id);
        return doc;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }
}


// ----- Write -----

// generates a new minutes document and stores it to the database
const newMin = async (newDoc, user) => {
    console.log(newDoc);
    console.log(user);
    var doc = {
        createDate: new Date(),
        modDate: new Date(),
        oid: ObjectId(newDoc.oid),
        title: newDoc.title,
        state: "open",
        creator: {
            uName: user.uName,
            fName: user.fName,
            lName: user.lName,
            uid: user._id,

        },
        contents: newDoc.contents
    }
    var resp = await dbconn.get().collection("minutes").insertOne(doc);
    return resp;
}



// ========================================
//             Express Routes
// ========================================

// middleware to log requests to this endpoint
router.use((req, res, next) => {
    console.log("REQ: MIN "+"IP: "+req.ip+" : "+req.url);
    next()
});

// ---- GET ----

// API endpoint that returns all minutes. 
// Included for demonstration purposes. No longer used.
// "./min/"
router.get('/', async (req, res) => {
    const doc = await listAll();
    res.json(doc);
});

// API endpoint to return minutes filtered by a given organization ID
// "./min/org/<OID>"
router.get('/org/:oid', async (req, res) => {
    console.log("MINUTES BY OID: "+req.params.oid);
    const resp = await minByOrg(req.params.oid);
    res.json(resp);
});

// API endpoint to return a specific minutes document by given document ID
// "./min/s/<ID>"
router.get('/s/:id', async (req, res) => {
    const resp = await minByID(req.params.id);
    res.json(resp);
});

// ---- POST ----
// API endpoint to store new minutes document
// "./min/new"
router.post('/new', validator.checkToken, validator.authUser, async (req, res) => {
    const resp = await newMin(req.body, req.user);
    console.log(resp);
    res.json(resp);
})

// EXPORTS
module.exports = {
    router
}