// USERS.JS
// Cole Cassidy - 2022
// Contains core user functions and routing

// Imports
const express = require('express');
const router = express.Router();

const { ObjectId } = require('mongodb');
const dbconn = require('./dbConn');
const iota = require('./iota');
const validator = require('./validator');

// ========================================
//              DB Functions
// ========================================

// ----- Read -----

// Returns user document matching provided user document ID.
const findUser = async (uid) => {
    try {
        const doc = await dbconn.get().collection("users").findOne({"_id": ObjectId(uid)});
        return doc;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }
}

// Returns a single document containing all registered users.
// Depricated. Included here for demonstration.
const listAllUsers = async () => {
    try {
        const doc = await dbconn.get().collection("users").find().toArray();
        return doc;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }
}

// Returns a user document matching provided username.
const findUserName = async (name) => {
    try {
        const doc = await dbconn.get().collection("users").findOne({"uName": name});
        return doc;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }
}

// ----- Write -----

// Generates a new user Document.
// This has been functionally replaced by the "regUser()" function in authentication.js
// Included here for admin access to add demonstration users.
const addUser = async (user) => {
    newUsr = {
        "uName": user.uName.toUpperCase(),
        "fName": user.fName,
        "lName": user.lName,
        "joindate": new Date()
    }
    var check = await findUserName(user.uName.toUpperCase());
    console.log(check);
    if(check != null) return iota.ERR.Unknown;
    try {
        var result = await dbconn.get().collection("users").insertOne(newUsr);
        result.document = newUsr;
        return result;
    } catch(e) {
        console.error(e);
        return iota.ERR.Unknown;
    }
}

// Modifies a user document given the new information and user document ID.
const editUser = async (user) => {
    const filter = {
        _id: new ObjectId(user._id)
    }
    const update = {
        $set: {
            'fName': user.fName,
            'lName': user.lName,
            'bio': user.bio
        }
    }
    const res = await dbconn.get().collection("users").updateOne(filter, update);
    return res;

}

// ========================================
//             Express Routes
// ========================================

// middleware that logs access to this endpoint.
router.use((req, res, next) => {
    console.log("REQ: USR  IP: "+req.ip+" : "+req.url);
    next();
});

// API endpoint to provide a list of all registered users.
// Included to demonstrate database functions at research presentations.
// Scheduled to be removed.
// "./usr/"
router.get('/', async (req, res) => {
    const doc = await listAllUsers();
    res.json(doc);
});

// API endpoint to return a specific user document.
// "./usr/s/<uid>"
router.get('/s/:uid', async (req, res) => {
    const doc = await findUser(req.params.uid);
    res.json(doc);
});

// API endpoint to return a list of all users.
// "./usr/list"
router.get('/list', async (req, res) => {
    const doc = await listAllUsers();
    res.json(doc);
});

// API endpoint to add a new User
// Deprecated. Included for demonstration purposes.
// "./usr/add"
router.post('/add', async (req, res) => {
    const doc = await addUser(req.body);
    res.json(doc);
});

// API endpoint to edit a userbio
// "./usr/edit/<uid>"
router.post('/edit/:uid', validator.checkToken, validator.authUser, async (req,res) => {
    const doc = await editUser(req.body);
    res.json(doc);
})

// EXPORTS
module.exports = {
    router,
    findUser
}